<?php
/**
 * Recording is an encapsulation of a MythTV recording entry.
 *
 * Recording objects represent a row from the MythTV recorded table.  These
 * rows represent an available recording in the MythTV system.
 *
 * @version $Id: Recording.php,v 1.14 2006/08/13 21:48:57 dbattams Exp $
 * @author Derek Battams <derek@battams.ca>
 * @package MRE
 * @copyright 2006 GPL v2
 */

/**#@+
 * Required include
 */
require_once(APP_ROOT . '/class/SysConf.php');
require_once(APP_ROOT . '/class/EventFactory.php');
require_once(APP_ROOT . '/class/RecordingRule.php');
/**#@-*/

/**
 * Encapsulates an available MythTV recording.
 *
 * @package MRE
 */
class Recording
{
   /**
    * @var array A hash representing a row from the recorded table in MythTV
    */
   private $_row;

   /**
    * @var MythEvent A concrete MythEvent monitor attached to monitor recording
    */
   private $_event;

   /**
    * @var bool If true, don't run the destructor
    */
   private $_disregard;

   /**
    * Class constructor.
    *
    * @param array A hash representing a row from the recorded table in MythTV
    */
   public function __construct($data)
   {
      $this->_disregard = FALSE;
      $this->_row = $data;
      $this->_event = EventFactory::getEvent($this->_row);
      return;
   }

   /**
    * Class destructor.
    *
    * The destructor processes the recording and the associated event monitor
    * and updates the recording time, if necessary.
    */
   public function __destruct()
   {
      if($this->_disregard)
         return;

      $conf = SysConf::getInstance();
      echo "    Title: " . $this->getVal('title') . "\n";
      echo " Subtitle: " . $this->getVal('subtitle') . "\n";
      echo "  Channel: " . $this->getVal('chanid') . "\n";
      echo "   Ending: " . $this->getVal('endtime') . "\nMonitored: ";
      if($this->_event && ($this->_event->isValid() || !$conf->getOpt('ignore')))
      {
         echo "Yes (" . get_class($this->_event) . ")\n   Status: ";
         $rule = new RecordingRule($this->getVal('recordid'));
         if($rule->getVal('type') != RecordingRule::RECTYPE_OVERRIDE && $rule->getVal('type') != RecordingRule::RECTYPE_SINGLE)
         {
            $override = clone $rule;
            $override->setVal('parentid', $rule->getVal('recordid'));
            $override->fillOverride($this);
            $rule = $override;
         }

         $now = time();
         $progend = $rule->getVal('enddate') . ' ' . $rule->getVal('endtime');
         $progend = strtotime($progend);
         $end = $progend + ($rule->getVal('endoffset') * 60);
         $timeLeft = intval(ceil(($end - $now) / 60));
         $offsetToNow = intval(ceil(($now - $progend) / 60));
         if($this->_event->isValid() && $this->_event->isEventOver())
         {
            if($now >= $progend || $conf->getOpt('compress'))
            {
               $rule->setVal('endoffset', $offsetToNow);
               $rule->setChanged(TRUE);
               echo "Event is over.  Recording will stop within 60 seconds.\n";
            }
            else
               echo "Event is over, but 'compress' set to false.\n";
         }
         else if($timeLeft <= $conf->getOpt('sleep'))
         {
            if(($end - $progend) < ($conf->getOpt('length') * 3600))
            {
               $rule->setVal('endoffset', $rule->getVal('endoffset') + $conf->getOpt('sleep'));
               $rule->setChanged(TRUE);
               echo "Event isn't over.  Recording extended.\n";
            }
            else
               echo "Event isn't over, but it's already been extended beyond the specified limit so doing nothing.\n";
         }
         else
            echo "Event isn't over.  Ignored until next check.\n";
         if(!$this->_event->isValid())
            echo "WARNING: Event monitor is invalid; assuming event isn't over.\n";
      }
      else
         echo "No\n";
      echo "\n";
      return;
   }

   /**
    * Get a DB column's value
    *
    * @param string Name of the column to get
    * @return string The value of $var or NULL if $var does not exist
    */ 
   public function getVal($var)
   {
      if(isset($this->_row[$var]))
         return $this->_row[$var];
      trigger_error("Invalid property!", E_USER_WARNING);
      return NULL;
   }

   /**
    * Set the disgregard flag for the object
    *
    * Set the disregard flag for the object, if true then the object's 
    * destructor is skipped
    * @var bool Value to set the flag to
    * @return void
    */
   public function disregard($val)
   {
      $this->_disregard = $val;
   }

   /**
    * Tells if the object has a valid event monitor attached to it.
    *
    * Returns true if the recording has a valid event monitor attached to it
    * or if it has no event monitor attached to it.  Returns false if there's
    * an attached event monitor, but it's invalid.
    * @return bool True if a valid event monitor is present or false if an invalid monitor is present (no event monitor is considered valid by this method)
    */
   public function hasValidEventMonitor()
   {
      return !is_object($this->_event) || $this->_event->isValid();
   }
}

?>
