<?php
/**
 * SysConf holds all configuration and common data for the application.
 * 
 * SysConf implements the singleton design pattern and holds all configuration
 * and common data for the application.  This includes the specified command
 * line options and the PEAR DB connection to the MythTV database.
 *
 * @version $Id: SysConf.php,v 1.16 2006/03/17 06:15:16 dbattams Exp $
 * @author Derek Battams <derek@battams.ca>
 * @package MRE
 * @copyright 2006 GPL v2
 */

/**#@+
 * Required include
 */
require_once('DB.php');
/**#@-*/

/**
 * A singleton holding the app's options and common data.
 *
 * @package MRE
 */
class SysConf
{
   /**
    * @var SysConf The lone instance of SysConf, all requests return this single one
    */
   static private $_instance = NULL;

   /**
    * Returns the lone instance of SysConf
    *
    * Implements the singleton design pattern to ensure that only one
    * SysConf object ever exists throughout the life of the applcation.
    *
    * @return SysConf The lone SysConf instance
    */
   static public function getInstance()
   {
      if(!self::$_instance)
         self::$_instance = new SysConf();
      return self::$_instance;
   }

   /**#@+
    * Debug level
    */
   const NORMAL  = 0;
   const ERROR   = 1;
   const WARNING = 2;
   const DEBUG   = 3;
   /**#@-*/

   /**
    * Path to the INI file
    */
   const INI_FILE = '/etc/mre.conf';

   /**
    * @var array Hash of options read from the command line
    */
   private $_opts;

   /**
    * @var DB A PEAR DB database connection object; connected to MythTV database
    */
   private $_db;

   /**
    * Class constructor.
    *
    * The SysConf constructor does some sanity checking, reads the command line,
    * and connects to the MythTV database.  Any fatal error halts the app.
    */
   protected function __construct()
   {
      if(php_sapi_name() != 'cli')
         trigger_error("Only the CLI SAPI is supported!", E_USER_ERROR);

      // Process the command line
      $cmdline = getopt('ho:tv');
      if($cmdline === FALSE)
         trigger_error("Can't parse command line!", E_USER_ERROR);
      $this->_opts = array(
	'help'		=> isset($cmdline['h']) ? TRUE : FALSE,
	'over'		=> isset($cmdline['o']) ? $cmdline['o'] : FALSE,
	'test'		=> isset($cmdline['t']) ? TRUE : FALSE,
        'version'	=> isset($cmdline['v']) ? TRUE : FALSE
	);

      if($this->getOpt('help'))
         printUsage();
      if($this->getOpt('version'))
         die(MRE_VERSION_INFO . "\n");

      $this->readINI();

      $this->_opts['dsn'] = "mysql://" . $this->_opts['dbuser'] . ':' . $this->_opts['dbpwd'] . '@' . $this->_opts['dbhost'] . '/' . $this->_opts['dbname'];
      if(!$this->getOpt('mythserv'))
         $this->_opts['mythserv'] = $this->getOpt('dbhost');

      if(!is_numeric($this->getOpt('sleep')) || $this->getOpt('sleep') < 2)
         trigger_error("-s must be greater than 2!", E_USER_ERROR);

      if($this->getOpt('log'))
      {
         $fp = @fopen($this->getOpt('log'), 'a');
         if($fp === FALSE)
            trigger_error("Unable to write to specified log!", E_USER_ERROR);
         fclose($fp);
         if(posix_geteuid() == 0)
         {
            if(!@chown($this->getOpt('log'), $this->getOpt('user')))
               trigger_error("Unable to change owner of log file!", E_USER_ERROR);
            $user = posix_getpwnam($this->getOpt('user'));
            if($user === FALSE)
               trigger_error("Can't run as root, but invalid user specified!", E_USER_ERROR);
            if(!posix_setgid($user['gid']) || !posix_setuid($user['uid']))
               trigger_error("Dropping root privs failed!", E_USER_ERROR);
         }
      }

      $this->_db = DB::connect($this->getOpt('dsn'));
      if(DB::isError($this->_db))
      {
         if($this->getOpt('verbose') >= self::DEBUG)
            echo $this->_db->getDebugInfo();
         else if($this->getOpt('verbose') >= self::ERROR)
            echo $this->_db->getMessage();
         trigger_error("Unable to connect to MythTV database!", E_USER_ERROR);
      }
      return;
   }

   /**
    * Class destructor
    */
   public function __destruct()
   {
      if(is_object($this->_db) && !DB::isError($this->_db))
         $this->_db->disconnect();
      return;
   }

   /**#@+
    * Getter
    */
   public function getOpt($var)
   {
      if(isset($this->_opts[$var]))
         return $this->_opts[$var];
      return NULL;
   }

   public function getDB()
   {
      return $this->_db;
   }
   /**#@-*/

   /**
    * Read the INI file and load data into singleton object
    *
    * Load the data from the INI into the singleton; also used as a callback
    * function for the SIGHUP signal handler, which will allow the reloading of
    * the INI file when the script receives the SIGHUP signal.  SIGHUP support
    * is only available if PHP is built with the pcntl_* functions.
    *
    * @param int Signal number, if called in context of a callback function
    */
   public function readINI($signo = NULL)
   {
      if(!is_readable(self::INI_FILE))
         trigger_error("Unable to read conf file!", E_USER_ERROR);
      $this->_opts = array_merge($this->_opts, parse_ini_file(self::INI_FILE));
      return;
   }
}

?>
